
;
;   events.inc
;
;   This file contains routines which handle interrupt events during main loop
;
;   last modified: 03/19/2002 mmddyy 09:44 by Burkart
;
; List of routines:
;
; - jmp handle_radio_logged_out
; - jmp handle_radio_ti_packet_sent
; - jmp handle_radio_packet_rcvd
;


;-----------------------------------------------
; jmp handle_radio_logged_out
;-----------------------------------------------
; interrupt handler set flag, this is the
; according "Event-Handler"
; TI-Server send {Logout ACK}
;-----------------------------------------------

handle_radio_logged_out:

IFDEF _DI_DURING_TIUC_COMM
	clr	IE.7					; DEVELOPMENT: disable interrupts
ENDIF

	; send <C logged out>:

	mov	dptr, #tiucpacket_uc_logged_out	; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI

	; receive <TI ACK>:

	call	wait_for_start_with_timeout		; wait for START
	call	receive_ti_packet			; receive packet

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; verify received packet
	jc	soft_reset				; packets different -> soft-reset

IFDEF _DI_DURING_TIUC_COMM
	setb	IE.7					; DEVELOPMENT: enable interrupts
ENDIF

	; [[[perform power-down actions]]]
	jmp	soft_reset				; DEVELOPMENT: soft reset

;-----------------------------------------------
; jmp handle_radio_ti_packet_sent
;-----------------------------------------------
; interrupt handler set flag, this is the
; according "Event-Handler"
; radio packet from TI has successfully been
; sent
;-----------------------------------------------

handle_radio_ti_packet_sent:
	clr	bit_radio_ti_packet_sent		; re-activate event

IFDEF _DI_DURING_TIUC_COMM
	clr	IE.7					; DEVELOPMENT: disable interrupts
ENDIF

	; send <C sent radio packet>:

	mov	dptr, #tiucpacket_uc_sent_radio_packet ; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI

	; receive <TI ACK>:

	call	wait_for_start_with_timeout		; wait for START
	call	receive_ti_packet			; receive packet

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; verify received packet
	jc	soft_reset				; packets different -> soft-reset

IFDEF _DI_DURING_TIUC_COMM
	setb	IE.7					; DEVELOPMENT: enable interrupts
ENDIF

	jmp	main_loop				; return to main_loop

;-----------------------------------------------
; jmp handle_radio_packet_rcvd
;-----------------------------------------------
; interrupt handler set flag, this is the
; according "Event-Handler"
; single radio packet with RCPT=me received and
; stored at ext_radio_packet_in
;-----------------------------------------------
; TODO: multi-packet support
;-----------------------------------------------

handle_radio_packet_rcvd:
	mov	dptr, #(ext_radio_packet_in+2)	; content-id; bit 7 set?
	movx	a, @dptr
	jb	ACC.7, ha_radio_pckt_rcvd_follow_packet

	; bit 7 of content-id cleared: first packet of (maybe multi-)packet

	; copy checksum with bit 7 (multi-packet indicator) cleared:
	anl	a, #01111111b
	mov	ram_radio_in_content_id, a

	clr	a					; reset ram_all_data_offset_hi & _lo to 0
	mov	ram_all_data_offset_hi, a
	mov	ram_all_data_offset_lo, a

	mov	dptr, #(ext_radio_packet_in+4)	; get multi-packet indicator with no. of total packets
	movx	a, @dptr
	mov	ram_multipckt_total_packets, a	; ...and save it

	mov	a, #1					; set number of current packet to 1
	mov	ram_multipckt_last_packet, a

	jmp	ha_radio_pckt_check_multipckt_complete ; check whether packet 1 was the only packet

ha_radio_pckt_rcvd_follow_packet:
	mov	dptr, #(ext_radio_packet_in+4)	; get multi-packet indicator with no. of current packet
	movx	a, @dptr

	clr	c
	subb	a, ram_multipckt_last_packet
	jz	ha_radio_packet_rcvd_nodata		; part-packet already received

;	inc	ram_multipckt_last_packet		; increase packet counter
	mov	dptr, #(ext_radio_packet_in+4)	; update packet counter
	movx	a, @dptr
	mov	ram_multipckt_last_packet, a

ha_radio_pckt_check_multipckt_complete:
	mov	a, ram_multipckt_last_packet	; last packet received?
	clr	c
	subb	a, ram_multipckt_total_packets

	clr	bit_radio_packet_rcvd_completely
	jnz	ha_radio_pckt_not_complete
	setb	bit_radio_packet_rcvd_completely	; yes, last packet received
ha_radio_pckt_not_complete:

	; get length of data into r2:

	mov	dptr, #(ext_radio_packet_in+1)
	movx	a, @dptr
	clr	c					; get data length only
	subb	a, #6
	jz	ha_radio_packet_rcvd_nodata		; data length=0 -> nothing to copy
	mov	r2, a

	; copy packet to all_data buffer:

	mov	a, #(LOW ext_radio_packet_all_data) ; initialize buffer pointers
	add	a, ram_all_data_offset_lo
	mov	r0, a
	mov	a, #(HIGH ext_radio_packet_all_data)
	addc	a, ram_all_data_offset_hi
	mov	P2, a
	mov	dptr, #(ext_radio_packet_in+5)

	mov	a, r2					; update offset to ext_radio_packet_all_data
	add	a, ram_all_data_offset_lo
	mov	ram_all_data_offset_lo, a
	clr	a
	addc	a, ram_all_data_offset_hi
	mov	ram_all_data_offset_hi, a

ha_radio_packet_rcvd_copyloop:
	movx	a, @dptr				; copy byte
	movx	@r0, a
	inc	dptr					; advance pointers
	inc	r0
	cjne	r0, #0, ha_radio_packet_rcvd_copyloop_next ; check for overflow
	inc	P2
ha_radio_packet_rcvd_copyloop_next:
	djnz	r2, ha_radio_packet_rcvd_copyloop	; go on until all data bytes are copied

ha_radio_packet_rcvd_nodata:

	clr	bit_radio_packet_rcvd		; re-activate event

IFDEF _DI_DURING_TIUC_COMM
	clr	IE.7					; DEVELOPMENT: disable interrupts
ENDIF

	; send <C received radio packet>:

	mov	dptr, #tiucpacket_uc_rcvd_radio_packet ; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI

	; receive <TI ACK>:

	call	wait_for_start_with_timeout		; wait for START
	call	receive_ti_packet			; receive packet

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; verify received packet
	jc	to5_soft_reset			; packets different -> soft-reset

	jnb	bit_radio_packet_rcvd_completely, to1x_main_loop

	; only go on if complete radio (multi-)packet has been received

IFDEF _SHOW_DATA_AFTER_EMPTY
	clr	bit_rcvd_radio_empty			; wait for {empty}
	jmp	main_loop				; return to main loop
ENDIF

ha_radio_packet_rcvd_send_rcvd_cpl_r_m_pckt:
	; send <C received complete radio (multi-)packet>

	mov	dptr, #tiucpacket_uc_rcvd_complete_radio_packet ; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI
	mov	a, ram_radio_in_content_id		; send Content-ID
	call	send_byte_to_ti
	mov	a, ram_all_data_offset_lo		; send data length
	call	send_byte_to_ti
	mov	a, ram_all_data_offset_hi
	call	send_byte_to_ti

	; receive <TI ACK>:

	call	wait_for_start_with_timeout		; wait for START
	call	receive_ti_packet			; receive packet

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; verify received packet
	jc	to5_soft_reset			; packets different -> soft-reset

IFDEF _DI_DURING_TIUC_COMM
	setb	IE.7					; DEVELOPMENT: enable interrupts
ENDIF

to1x_main_loop:
	jmp	main_loop				; return to main loop

to5_soft_reset:
	jmp	soft_reset


IFDEF _SHOW_DATA_AFTER_EMPTY

;-----------------------------------------------
; jmp handle_radio_empty_rcvd
;-----------------------------------------------
; interrupt handler set flag, this is the
; according "Event-Handler"
; single radio packet with RCPT=me received and
; stored at ext_radio_packet_in, afterwards
; {empty} received -> server received {ACK}
;-----------------------------------------------

handle_radio_empty_rcvd:
IFDEF _DI_DURING_TIUC_COMM
	clr	IE.7					; DEVELOPMENT: disable interrupts
ENDIF
	jbc	bit_radio_packet_rcvd_completely, ha_radio_packet_rcvd_send_rcvd_cpl_r_m_pckt
	clr	bit_rcvd_radio_empty
	jmp	main_loop

ENDIF