
;
;   handleti.inc
;
;   This file contains routines to handle START and <TI ...> packets during main loop
;
;   last modified: 03/14/2002 mmddyy 23:32 by Burkart
;
; List of routines:
;
; - jmp main_handle_start
; - jmp main_reply_to_ping
; - jmp main_reply_to_ti_exit
; - jmp main_reply_to_ti_send_radio_packet
; - jmp main_reply_to_ti_request_radio_packet
;


;-----------------------------------------------
; jmp main_handle_start
;-----------------------------------------------
; main_loop received START, this routine handles
; it, receives the packet and replies according
; to what packet has been received
;-----------------------------------------------

main_handle_start:
IFDEF _DI_DURING_TIUC_COMM
	clr	IE.7					; DEVELOPMENT: disable interrupts
ENDIF

	call	wait_for_start_no_timeout		; handle START
	call	receive_ti_packet			; receive packet

	; check received packet for <TI ping>:

	mov	dptr, #tiucpacket_ping		; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; check received packet
	jnc	main_reply_to_ping			; TI sent <TI ping> -> reply with <C ACK>

	; check received packet for <TI exit>:

	mov	dptr, #tiucpacket_ti_exit		; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; check received packet
	jnc	main_reply_to_ti_exit		; TI sent <TI exit> -> reply

	; check received packet for <TI send radio packet>:

	mov	P2, #(HIGH ext_tiuc_packet_in)	; standard check_received_packet won't do
	mov	r0, #(LOW ext_tiuc_packet_in + 2)
	movx	a, @r0
	clr	c
	subb	a, #40h
	jz	main_reply_to_ti_send_radio_packet ; TI sent <TI send radio packet> -> reply

	; check received packet for <TI request radio packet>:

	mov	dptr, #tiucpacket_ti_request_radio_packet ; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; check received packet
	jnc	to1_main_reply_to_ti_request_radio_packet ; TI sent <TI request radio packet> -> reply

	; TI sent some packet, C can't handle

	jmp	main_loop				; jump back to main_loop

to1_main_reply_to_ti_request_radio_packet:
	jmp	main_reply_to_ti_request_radio_packet

;-----------------------------------------------
; jmp main_reply_to_ping
;-----------------------------------------------
; during main_loop C received <TI ping>
; this routines replies according to TI<->C HLP
; and jumps back to main_loop
;-----------------------------------------------

main_reply_to_ping:
	; send <C ACK>:

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI

	jmp	main_loop

;-----------------------------------------------
; jmp main_reply_to_ti_exit
;-----------------------------------------------
; during main_loop C received <TI exit>
; this routines replies according to TI<->C HLP
; by performing logout procedure 1 (TI shutdown)
;-----------------------------------------------

main_reply_to_ti_exit:
	; send <C ACK>:

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI

IFDEF _DI_DURING_TIUC_COMM
	setb	IE.7					; enable interrupts
ENDIF

main_reply_to_ti_exit_loop1:
	jb	bit_radio_ti_packet_pending, main_reply_to_ti_exit_loop1 ; wait until TI's packet has been sent

	mov	dptr, #ext_radio_packet_out		; assemble {Client logout request} at ext_radio_packet_out
	mov	r2, #0					; initialize checksum
	mov	a, #1					; RCPT = Server = 1
	call	calc_crc_and_store
	mov	a, #6					; Length = 6
	call	calc_crc_and_store
	mov	a, #3					; Content-ID = 3 = Logout request
	call	calc_crc_and_store
	mov	a, ram_radio_address			; Sender = me
	call	calc_crc_and_store
	mov	a, #1					; multi-packet data = 1 (packet count)
	call	calc_crc_and_store
	mov	a, r2					; checksum (method: CRC8)
	movx	@dptr, a

	setb	bit_radio_ti_packet_pending		; advise interrupt handler to send packet

	jmp	main_loop				; return to main loop (which will then get a radio_logged_out event)

;-----------------------------------------------
; jmp main_reply_to_ti_send_radio_packet
;-----------------------------------------------
; during main_loop C received <TI send radio
; packet>
; this routines replies according to TI<->C HLP
;-----------------------------------------------
; TODO: give TI the ability to send multi-packets
;-----------------------------------------------

main_reply_to_ti_send_radio_packet:
	; send <C ACK>:

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI

	mov	P2, #(HIGH ext_tiuc_packet_in)	; init data pointers
	mov	r0, #(LOW ext_tiuc_packet_in)
	mov	dptr, #ext_radio_packet_out

	movx	a, @r0					; get data length into r3
	dec	a
	dec	a
	mov	r3, a

; +3 content-id
; +4.. data

	mov	r2, #0					; init checksum

	mov	a, #01h				; RCPT = Server = 01h (always)
	call	crc8_add_to_r2
	movx	@dptr, a
	inc	dptr
	mov	a, r3					; length = data length + 6
	add	a, #6
	call	crc8_add_to_r2
	movx	@dptr, a
	inc	dptr
	inc	r0					; Content-ID, see TI(<->C) packet
	inc	r0
	inc	r0
	movx	a, @r0
	call	crc8_add_to_r2
	movx	@dptr, a
	inc	dptr
	mov	a, ram_radio_address			; sender = me
	call	crc8_add_to_r2
	movx	@dptr, a
	inc	dptr
	mov	a, #1					; multi-packet (packet count)
	call	crc8_add_to_r2
	movx	@dptr, a
	inc	dptr

	mov	a, r3					; check data length for 0
	jz	main_reply_to_ti_send_radio_p_nodata

main_reply_to_ti_send_radio_p_loop:
	inc	r0					; advance buffer pointer to next byte
	movx	a, @r0					; read byte
	call	crc8_add_to_r2			; add it to checksum
	movx	@dptr, a				; copy it
	inc	dptr					; advance buffer pointer
	djnz	r3, main_reply_to_ti_send_radio_p_loop ; copy all data bytes

main_reply_to_ti_send_radio_p_nodata:
	; now only the checksum byte (which is in r2) is left

	mov	a, r2					; add checksum byte
	movx	@dptr, a

	setb	bit_radio_ti_packet_pending		; advise interrupt handler to send TI radio packet

	jmp	main_loop				; return to main_loop

;-----------------------------------------------
; jmp main_reply_to_ti_request_radio_packet
;-----------------------------------------------
; during main_loop C received <TI request radio
; packet>
; this routines replies according to TI<->C HLP
;-----------------------------------------------

main_reply_to_ti_request_radio_packet:
	mov	P2, #(HIGH (ext_tiuc_packet_in+6)) ; initialize buffer pointer
	mov	r0, #(LOW (ext_tiuc_packet_in+6))

	movx	a, @r0					; get hi-length
	mov	r2, a					; and store it in r2
	dec	r0					; get lo-length
	movx	a, @r0
	mov	r3, a					; and store it in r3
	dec	r0					; get lo-offset
	dec	r0
	movx	a, @r0
	add	a, #(LOW ext_radio_packet_all_data) ; add it to start address
	mov	DPL, a					; and store it in DPL
	inc	r0					; get hi-offset
	movx	a, @r0
	addc	a, #(HIGH ext_radio_packet_all_data) ; add it to start address with carry from addition above
	mov	DPH, a					; and store it in DPH

	call	send_start				; send START to TI
	mov	a, #2					; send packet length (= data length+2)
	add	a, r3
	call	send_byte_to_ti
	clr	a
	addc	a, r2
	call	send_byte_to_ti
	mov	a, #54h				; send packet identifier for <C send radio packet>
	call	send_byte_to_ti
	mov	a, ram_radio_in_content_id		; send Content-ID of radio packet
	call	send_byte_to_ti

	mov	a, r2					; if high-length=0 then don't send hi*256 byte (this would be 64kB)
	jz	main_reply_to_ti_request__loop3

main_reply_to_ti_request__loop1:
	mov	r0, #0
main_reply_to_ti_request__loop2:
	movx	a, @dptr				; get byte from ext_radio_packet_all_data
	call	send_byte_to_ti			; send it to TI
	inc	dptr					; advance buffer pointer
	djnz	r0, main_reply_to_ti_request__loop2
	djnz	r2, main_reply_to_ti_request__loop1

main_reply_to_ti_request__loop3:
	movx	a, @dptr				; get byte from ext_radio_packet_all_data
	call	send_byte_to_ti			; send it to TI
	inc	dptr					; advance buffer pointer
	djnz	r3, main_reply_to_ti_request__loop3

	; receive <TI ACK>:

	call	wait_for_start_with_timeout		; wait for START
	call	receive_ti_packet			; receive packet

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; verify received packet
	jc	to6_soft_reset			; packets different -> soft-reset

	jmp	main_loop				; return to main_loop

to6_soft_reset:
	jmp	soft_reset

