
;
;   radlogin.inc
;
;   This file contains code for the radio login procedure
;
;   last modified: 03/17/2002 mmddyy 15:04 by Burkart
;


radio_login_start:					; warten auf Anmeldefreigabe
	clr	LED_PIN				; DEBUG: show status
	mov	dptr, #radiopacket_anmeldefreigabe
	mov	r0, #0

serial_in1:
	jnb	SCON.0, serial_in1			; auf empfangenes Byte warten
	mov	r1, SBUF				; wenn da, auf R1 ausgeben
	clr	SCON.0					; Empfang weiterer Bytes ein
	; empfangenes Byte in r1

	jnb	SCON.2, radio_login_start		; Stopbit OK? Nein -> alles nochmal von vorne
	; Stopbit OK

	mov	a, r0					; a = Packet offset
	movc	a, @a+dptr				; a = Packet data
	clr	c					; empfangenes Byte = wie es sein soll?
	subb	a, r1
	jnz	radio_login_start			; nein -> alles nochmal von vorne

	inc	r0					; Packet offset++
	cjne	r0, #6, serial_in1			; alle 6 Byte empfangen? Nein -> nchstes Byte
	; Anmeldefreigabe empfangen

IFDEF _TXD_DEFAULT_LO
	setb	TXD
ENDIF
	setb	TXEN_PIN				; TXEN auf Senden setzen
	mov	a, #30					; 10ms warten
	call	wait_a_ms

	mov	a, #0FFh				; send FFh in order to gain bit synchronization
	call	serial_out
	mov	a, #START_CHAR			; send START-char
	call	serial_out

	mov	r2, #0					; checksum initialisieren

	mov	a, #01h				; recipient: 01h (TI-Server)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, #13h				; length: 13h=19d
	call	crc8_add_to_r2
	call	serial_out
	mov	a, #00h				; Content-ID: 00h (???)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, #0FFh				; sender: FFh
	call	crc8_add_to_r2
	call	serial_out
	mov	a, #01h				; Multi-Packet: 01h (packet count)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, ram_login_name			; login name: "????????"
	call	crc8_add_to_r2
	call	serial_out
	mov	a, (ram_login_name+1)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, (ram_login_name+2)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, (ram_login_name+3)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, (ram_login_name+4)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, (ram_login_name+5)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, (ram_login_name+6)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, (ram_login_name+7)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, ram_calc_type			; calc type: 82h/83h
	call	crc8_add_to_r2
	call	serial_out
	mov	a, #HI_VER				; C-software hi-version
	call	crc8_add_to_r2
	call	serial_out
	mov	a, #LO_VER				; C-software lo-version
	call	crc8_add_to_r2
	call	serial_out
	mov	a, ram_ti_hi_version			; TI-software hi-version
	call	crc8_add_to_r2
	call	serial_out
	mov	a, ram_ti_lo_version			; TI-software lo-version
	call	crc8_add_to_r2
	call	serial_out
	mov	a, r2					; checksum: ??h (method: CRC8)
	call	serial_out
	call	wait_txempty				; wait until last byte has been sent

	clr	TXEN_PIN				; TXEN auf Empfangen setzen
IFDEF _TXD_DEFAULT_LO
	clr	TXD
ENDIF
	mov	a, #3					; 3ms warten
	call	wait_a_ms

	; Vom TI-Server zugewiesene Adresse empfangen:

	setb	LED_PIN				; DEBUG: show status
	mov	r3, #(3+1)				; at least the third byte=START-char has to be the START-char
							; of {Login ACK}, otherwise the whole login is repeated

retrieve_radio_address_start:			; warten auf Anmeldebesttigung
	dec	r3
	cjne	r3, #0, retrieve_radio_address_start2
	jmp	radio_login_start
retrieve_radio_address_start2:
	mov	dptr, #radiopacket_anmeldebestaetigung
	mov	r0, #0
	mov	ram_checksum, #0

serial_in2a:						; wait for START-char with correct stopbit
	call	serial_in
	jnb	SCON.2, serial_in2a
	cjne	a, #START_CHAR, serial_in2a

serial_in2:
	call	serial_in				; receive serial byte -> A
	mov	r1, a
	; empfangenes Byte in r1

	jnb	SCON.2, retrieve_radio_address_start ; Stopbit OK? Nein -> alles nochmal von vorne
	; Stopbit OK

	mov	a, r0					; a = Packet offset
	movc	a, @a+dptr				; a = Packet data
	call	crc8_add
	clr	c					; empfangenes Byte = wie es sein soll?
	subb	a, r1
	jnz	retrieve_radio_address_start	; nein -> alles nochmal von vorne

	inc	r0					; Packet offset++
	cjne	r0, #5, serial_in2			; first 5 byte have to be identical

	; Anmeldebesttigung wahrscheinlich empfangen (zumindest entsprechenden Header)

	mov	r0, #ram_login_name
serial_in3:
	call	serial_in				; receive serial byte -> A
	mov	r1, a
	; empfangenes Byte in r1

	jnb	SCON.2, retrieve_radio_address_start ; Stopbit OK? Nein -> alles nochmal von vorne
	; Stopbit OK

	mov	a, @r0					; get byte of username
	call	crc8_add
	clr	c					; equals received byte?
	subb	a, r1
	jnz	retrieve_radio_address_start	; nein -> alles nochmal von vorne

	inc	r0					; username offset++
	cjne	r0, #(ram_login_name+8), serial_in3 ; 8 byte have to be identical

	; retrieve assigned radio address:

	call	serial_in				; serielles Byte empfangen -> A

	jnb	SCON.2, retrieve_radio_address_start ; Stopbit OK? Nein -> alles nochmal von vorne
	; Stopbit OK

	call	crc8_add
	mov	ram_radio_address, a			; store assigned address

	; verify checksum

	call	serial_in				; serielles Byte empfangen -> A

	jnb	SCON.2, retrieve_radio_address_start ; Stopbit OK? Nein -> alles nochmal von vorne
	; Stopbit OK

	mov	r1, ram_checksum			; verify checksum
	clr	c
	subb	a, r1
	jnz	retrieve_radio_address_start	; different -> again from the beginning

	; now this UU has an assigned address (see ram_radio_address)

IFDEF _STANDARD_ACK_LOGIN
	; send {ACK}:

	mov	r2, #0					; initialize CRC
	mov	a, #1					; RCPT = 1 = Server
	call	crc8_add_to_r2
	call	serial_out
	mov	a, #6					; length = 6
	call	crc8_add_to_r2
	call	serial_out
	mov	a, #1					; Content-ID = 1
	call	crc8_add_to_r2
	call	serial_out
	mov	a, ram_radio_address			; sender = me
	call	crc8_add_to_r2
	call	serial_out
	mov	a, #1					; multi-packet = 1 (packet count)
	call	crc8_add_to_r2
	call	serial_out
	mov	a, r2					; checksum = ??? (method: CRC8)
	call	serial_out
ENDIF

IFNDEF _DI_DURING_TIUC_COMM
	setb	IE.4					; enable serial interrupt
ENDIF

