
;
;   serialio.inc
;
;   This file contains routines for serial I/O
;
;   last modified: 03/14/2002 mmddyy 23:18 by Burkart
;
; List of routines:
;
; - call serial_out
; - call wait_txempty
; - call serial_in
; - call serial_in_nowait
;


;-----------------------------------------------
; call serial_out
;-----------------------------------------------
; Sendet ein Byte in A via UART
; Wartet im Fall des Falles, bis das vorherige
; Byte gesendet wurde (Sendepuffer leer)
;-----------------------------------------------
; modifiziert: nix
;-----------------------------------------------
; stack: caller address
;-----------------------------------------------

serial_out:
	jnb	SCON.1, serial_out			; warten, bis bereit zu senden
	clr	SCON.1
	mov	SBUF, a 				; senden
	ret						; zurck zum Aufrufer

;-----------------------------------------------
; call wait_txempty
;-----------------------------------------------
; wait until last byte has been sent
;-----------------------------------------------
; modified: N/A
;-----------------------------------------------
; stack: caller address
;-----------------------------------------------

wait_txempty:
	jnb	SCON.1, wait_txempty
	ret


;-----------------------------------------------
; call serial_in
;-----------------------------------------------
; Empfngt ein Byte via UART
; Wartet im Fall des Falles, bis ein Byte
; empfangen wurde
;-----------------------------------------------
; output: A - received byte
;-----------------------------------------------
; modifiziert: nix
;-----------------------------------------------
; stack: caller address
;-----------------------------------------------

serial_in:
	jnb	SCON.0, serial_in			; warten, bis Byte da
	mov	a, SBUF				; empfangenes Byte abrufen
	clr	SCON.0					; bereit zum Empfang weiterer Bytes
	ret

;-----------------------------------------------
; call serial_in_nowait
;-----------------------------------------------
; Empfngt ein Byte via UART
; Steht kein Byte zum Empfang bereit, kehrt die
; Routine sofort zum Aufrufer zurck
;-----------------------------------------------
; output: CY=0: A - received byte
;         CY=1: N/A (no byte received)
;-----------------------------------------------
; modifiziert: CY
;-----------------------------------------------
; stack: caller address
;-----------------------------------------------

serial_in_nowait:
	jnb	SCON.0, serial_in_nowait_label0	; war ein Byte angekommen?
	mov	a, SBUF				; ja, abrufen
	clr	SCON.0					; Empfang weiterer Bytes ermglichen
	clr	c					; CY=0
	ret						; zurck zum Aufrufer
serial_in_nowait_label0:				; kein Byte angekommen
	setb	c					; CY=1
	ret						; zurck zum Aufrufer

