
;
;   serint.inc
;
;   This file contains the serial interrupt handler
;
;   last modified: 03/16/2002 mmddyy 01:15 by Burkart
;


;===========================================================
; Interruptbehandlungsroutine serielle Schnittstelle
;===========================================================
; 2+4 Byte Stack
;===========================================================
; Fall 1: TI=1: Zeichen wurde gesendet
;-----------------------------------------------------------
; Ist ram_byte_to_send = 0, werden keine weiteren Daten
; gesendet, TI bleibt = 1, ram_send_offset wird = 0 gesetzt.
;
; Ist ram_byte_to_send <> 0, wird das Byte an der Stelle
; ext_radio_packet_out_int+ram_send_offset im externen SRAM
; gesendet.
; ram_byte_to_send--; ram_send_offset++
;===========================================================
; Fall 2: TI=0 (RI=1): Zeichen wurde empfangen
;-----------------------------------------------------------
; [noch nix]
;===========================================================

 ORG 23h
serial_int:
IFDEF _USER_LED_ON_SER_INT
	setb	LED_PIN
ENDIF

	push	psw					; save PSW
	push	dph					; save used registers
	push	dpl
	push	acc
	push	0
	push	1
	push	2
	push	3
	push	4
	push	5
	push	6
	push	7

	jb	SCON.0, serial_int_ri		; RI?
	jb	SCON.1, to1_serial_int_ti		; TI?
	; shouldn't be possible to get here, should it?
to1x_ser_int_end:
	jmp	ser_int_end
to1_serial_int_ti:
	jmp	serial_int_ti				; TI

serial_int_ri:
	; === RI ===

	mov	a, SBUF				; empfangenes Byte abrufen
	clr	SCON.0					; bereit zum Empfang weiterer Bytes

	jb	SCON.2, ser_int_stopbit_ok		; Stopbit OK? Nein -> alles nochmal von vorne
	jmp	ser_int_framing_error
ser_int_stopbit_ok:
	; Stopbit OK

	jnb	bit_int_awaiting_start, ser_int_packet_byte ; already received START-char?
	; no, awaiting START-char
	cjne	a, #START_CHAR, to1x_ser_int_end	; not START-char -> end interrupt handler
	clr	bit_int_awaiting_start		; START-char
	jmp	ser_int_framing_error		; -> reset pointers

ser_int_packet_byte:
	mov	r3, a					; store received byte at ext_radio_packet_last_in+ram_radio_packet_l_in_offset
	mov	dptr, #ext_radio_packet_last_in
	mov	a, ram_radio_packet_l_in_offset
	add	a, DPL
	mov	DPL, a
	clr	a
	addc	a, DPH
	mov	DPH, a
	mov	a, r3
	movx	@dptr, a

	mov	a, ram_radio_packet_l_in_offset	; has length byte been received?
	dec	a
	jnz	ser_int_not_length_byte

	; length byte has been received

	mov	a, r3					; ram_radio_packet_l_in_remaining = length(-byte)-1
	dec	a
	mov	ram_radio_packet_l_in_remaining, a

ser_int_not_length_byte:
	inc	ram_radio_packet_l_in_offset	; update pointer and remaining-byte-counter
	dec	ram_radio_packet_l_in_remaining
	mov	a, ram_radio_packet_l_in_remaining

	jnz	to1_ser_int_end			; ram_radio_packet_l_in_remaining != 0 -> RETI

	; received radio packet at ext_radio_packet_last_in

	mov	r3, ram_radio_packet_l_in_offset	; get length of packet (without checksum byte) into r3
	dec	r3

	mov	ram_radio_packet_l_in_offset, #0	; update pointer
	setb	bit_int_awaiting_start		; next packet has to start with START-char

	; received radio packet at ext_radio_packet_last_in

	mov	r2, #0					; calculate checksum of packet at ext_radio_packet_last_in
	mov	dptr, #ext_radio_packet_last_in
ser_int_calc_checksum:
	movx	a, @dptr
	inc	dptr
	call	crc8_add_to_r2
	djnz	r3, ser_int_calc_checksum

	movx	a, @dptr				; load checksum byte of received packet
	clr	c					; compare to calculated checksum
	subb	a, r2
	jz	ser_int_checksum_ok			; different -> RETI
to1_ser_int_end:
	jmp	ser_int_end

ser_int_checksum_ok:
	; checksum is OK, most probably packet was received correctly

IFDEF _USER_LED_ON_CHECKSUM_OK
	setb	LED_PIN
ENDIF

	mov	dptr, #ext_radio_packet_last_in	; RCPT=me?
	movx	a, @dptr
	clr	c
	subb	a, ram_radio_address
	jz	ser_int_rcpt_is_me
	jmp	ser_int_rcpt_not_me			; no, but maybe sender=me-1
ser_int_rcpt_is_me:

	mov	dptr, #(ext_radio_packet_last_in+3) ; Sender=Server=1?
	movx	a, @dptr
	dec	a
	jnz	to1_ser_int_end			; no -> RETI

	mov	dptr, #(ext_radio_packet_last_in+2) ; Content-ID = 1 = {ACK}?
	movx	a, @dptr
	mov	r2, a
	dec	a
	jnz	ser_int_no_ack_received
	jmp	ser_int_ack_received
ser_int_no_ack_received:

	mov	a, r2					; Content-ID = 5 = {NACK}?
	clr	c
	subb	a, #5
	jz	to1_ser_int_end

	mov	a, r2					; Content-ID = 4 = {Sendefreigabe}?
	clr	c
	subb	a, #4
	jz	ser_int_send_packet

	mov	a, r2					; Content-ID = 10 = {empty}
	clr	c
	subb	a, #10
	jz	ser_int_empty

	mov	a, r2					; Content-ID = 8 = {logout ACK}
	clr	c
	subb	a, #8
	jz	ser_int_logout_ack

;	mov	a, r2					; Content-ID = 9 = {new address}
;	clr	c
;	subb	a, #9
;	jz	ser_int_new_address

	; it was a data packet

	jnb	bit_radio_packet_rcvd, ser_int_buffer_empty ; make sure, ext_radio_packet_in is empty
	; buffer overflow -> set response type {NACK}

	setb	bit_radio_reply_nack			; next time sending: send {NACK}
to2x_ser_int_end:
	jmp	ser_int_end				; RETI

ser_int_empty:
	setb	bit_rcvd_radio_empty
	jmp	to2x_ser_int_end

ser_int_logout_ack:
	setb	bit_radio_logged_out
	jmp	to2x_ser_int_end

ser_int_buffer_empty:
	; copy ext_radio_packet_last_in to ext_radio_packet_in:
	push	P2					; save P2 of main program
	mov	P2, #(HIGH ext_radio_packet_in)	; set pointer
	mov	r0, #(LOW ext_radio_packet_in)
	mov	dptr, #(ext_radio_packet_last_in+1) ; get length of packet
	movx	a, @dptr
	mov	r2, a
	mov	dptr, #ext_radio_packet_last_in	; set pointer
ser_int_copy_received_packet:
	movx	a, @dptr				; copy byte-wise
	movx	@r0, a
	inc	dptr					; update pointers
	inc	r0
	cjne	r0, #0, ser_int_copy_received_packet_next
	; overflow
	inc	P2
ser_int_copy_received_packet_next:
	djnz	r2, ser_int_copy_received_packet	; go on until complete packet is copied
	pop	P2					; restore P2 of main program

	setb	bit_radio_packet_rcvd		; tell main program that received packet is at ext_radio_packet_in
	setb	bit_radio_reply_ack			; next time sending: send {ACK}

	jmp	ser_int_end				; RETI

ser_int_send_packet:
	jb	bit_radio_reply_nack, ser_int_send_nack ; send {NACK}?
	jb	bit_radio_reply_ack, ser_int_send_ack ; send {ACK}?
	jb	bit_radio_ti_packet_pending, ser_int_to1x_send_tipacket ; send TI's packet?
	jmp	ser_int_send_empty
ser_int_to1x_send_tipacket:
	jmp	ser_int_send_tipacket
ser_int_send_empty:

	; send {empty}:
	mov	dptr, #(ext_radio_packet_out_int)	; assemble packet at ext_radio_packet_out_int
	mov	r2, #0					; initialize CRC
	mov	a, #1					; RCPT = 1 = Server
	call	calc_crc_and_store
	mov	a, #6					; length = 6
	call	calc_crc_and_store
	mov	a, #10					; Content-ID = 10
	call	calc_crc_and_store
	mov	a, ram_radio_address			; sender = me
	call	calc_crc_and_store
	mov	a, #1					; multi-packet = 1 (packet count)
	call	calc_crc_and_store
	mov	a, r2					; checksum = ??? (method: CRC8)
	movx	@dptr, a

	mov	ram_radio_byte_to_send, #6		; packet is 6 byte long

ser_int_finally_send_packet:
;IFDEF _TXD_DEFAULT_LO
;	clr	TXD
;ENDIF
	setb	TXEN_PIN				; TXEN = transmit
	mov	a, #8					; wait 8ms (for server's conveniance, usually 3ms)
	call	wait_a_ms
IFDEF _TXD_DEFAULT_LO
	setb	TXD
ENDIF

	mov	a, #0FFh				; send FFh in order to guarantee bit-synchronization
	call	serial_out
	mov	a, #START_CHAR			; send START-char
	call	serial_out

	mov	ram_radio_byte_to_send_offset, #0	; set offset to first byte of packet

	jmp	serial_int_send			; simulate TI interrupt

ser_int_send_nack:
	; send {NACK}:
	clr	bit_radio_reply_nack

	mov	dptr, #ext_radio_packet_out_int	; assemble packet at ext_radio_packet_out_int
	mov	r2, #0					; initialize CRC
	mov	a, #1					; RCPT = 1 = Server
	call	calc_crc_and_store
	mov	a, #6					; length = 6
	call	calc_crc_and_store
	mov	a, #5					; Content-ID = 5
	call	calc_crc_and_store
	mov	a, ram_radio_address			; sender = me
	call	calc_crc_and_store
	mov	a, #1					; multi-packet = 1 (packet count)
	call	calc_crc_and_store
	mov	a, r2					; checksum = ??? (method: CRC8)
	movx	@dptr, a

	mov	ram_radio_byte_to_send, #6		; packet is 6 byte long

	jmp	ser_int_finally_send_packet		; finally send it

ser_int_send_ack:
	; send {ACK}:
	clr	bit_radio_reply_ack

	mov	dptr, #ext_radio_packet_out_int	; assemble packet at ext_radio_packet_out_int
	mov	r2, #0					; initialize CRC
	mov	a, #1					; RCPT = 1 = Server
	call	calc_crc_and_store
	mov	a, #6					; length = 6
	call	calc_crc_and_store
	mov	a, #1					; Content-ID = 1
	call	calc_crc_and_store
	mov	a, ram_radio_address			; sender = me
	call	calc_crc_and_store
	mov	a, #1					; multi-packet = 1 (packet count)
	call	calc_crc_and_store
	mov	a, r2					; checksum = ??? (method: CRC8)
	movx	@dptr, a

	mov	ram_radio_byte_to_send, #6		; packet is 6 byte long

	jmp	ser_int_finally_send_packet		; finally send it

ser_int_send_tipacket:
	; send {TI's packet}:

	; first copy ext_radio_packet_out to set_radio_packet_out_int:
	mov	dptr, #(ext_radio_packet_out+1)	; get packet length into r2
	movx	a, @dptr
	mov	r2, a
	mov	ram_radio_byte_to_send, a
	mov	dptr, #(ext_radio_packet_out)
	push	P2
	mov	P2, #(HIGH ext_radio_packet_out_int)
	mov	r0, #(LOW ext_radio_packet_out_int)

ser_int_copy_tipacket:
	movx	a, @dptr				; copy TI packet from ext_radio_packet_out to ext_radio_packet_out_int
	movx	@r0, a
	inc	dptr					; increase pointers
	inc	r0
	cjne	r0, #0, ser_int_copy_tipacket_next ; check for overflow
	inc	P2
ser_int_copy_tipacket_next:
	djnz	r2, ser_int_copy_tipacket

	pop	P2

	jmp	ser_int_finally_send_packet		; finally send it

ser_int_rcpt_not_me:
	; RCPT not me
	; Sender = me-1?

	mov	dptr, #(ext_radio_packet_last_in+3)
	movx	a, @dptr
	inc	a
	clr	c
	subb	a, ram_radio_address
	jnz	ser_int_end				; sender != me-1 -> RETI
	jmp	ser_int_send_packet			; sender == me-1 -> send packet

ser_int_ack_received:
	; {ACK} received

	clr	bit_radio_ti_packet_pending
	setb	bit_radio_ti_packet_sent
	jmp	ser_int_end

ser_int_framing_error:
	; wrong stopbit -> abandon complete packet

IFDEF _USER_LED_ON_CHECKSUM_OK
	clr	LED_PIN
ENDIF

	mov	ram_radio_packet_l_in_offset, #0	; reset buffer pointers
	mov	ram_radio_packet_l_in_remaining, #0

	jmp	serial_int_end			; quit interrupt handler

serial_int_ti:
	; === TI ===

	clr	a					; sind noch Daten zu senden?
	cjne	a, ram_radio_byte_to_send, serial_int_send ; ja -> senden

	; ram_radio_byte_to_send=0  <=>  keine Daten mehr zu senden
	mov	ram_radio_byte_to_send_offset, #0
	clr	TXEN_PIN				; TXEN = receive
IFDEF _TXD_DEFAULT_LO
	clr	TXD
ENDIF
	jmp	serial_int_end			; Interrupt handler beenden

serial_int_send:
	; noch ram_radio_byte_to_send Byte Daten zu senden

	mov	a, #(LOW ext_radio_packet_out_int)	; initialize buffer pointer
	add	a, ram_radio_byte_to_send_offset
	mov	dpl, a
	clr	a
	addc	a, #(HIGH ext_radio_packet_out_int)
	mov	dph, a

	movx	a, @dptr				; zu sendendes Byte lesen

	call	serial_out				; Byte senden, wenn bereit

	dec	ram_radio_byte_to_send		; ein Byte weniger zu senden
	inc	ram_radio_byte_to_send_offset	; beim nchsten Mal das nchste Byte senden

serial_int_end:
ser_int_end:
	pop	7
	pop	6
	pop	5
	pop	4
	pop	3
	pop	2
	pop	1
	pop	0
	pop	acc					; restore saved registers
	pop	dpl
	pop	dph
	pop	psw					; restore PSW

IFDEF _USER_LED_ON_SER_INT
	clr	LED_PIN
ENDIF
	reti						; return from interrupt

