
;
;   supply.inc
;
;   This file contains some supply routines
;
;   last modified: 03/14/2002 mmddyy 23:13 by Burkart
;
; List of routines:
;
; - call wait_1sec
; - call wait_a_ms
; - call crc8_add
; - call crc8_add_to_r2
; - call calc_crc_and_store
; - call debug_wait
;


;-----------------------------------------------
; call wait_1sec
;-----------------------------------------------
; modifiziert: N/A
;-----------------------------------------------
; Stack: 4 Byte + caller address
;-----------------------------------------------
; Warteschleife: < 997 ms
;  Anzahl Maschinenzyklen: < 918868.4333333332

wait_1sec:
	push	PSW
	push	0
	push	1
	push	2
	mov	2, #60
wait_1sec_labelC2:
	mov	1, #87
wait_1sec_labelC1:
	mov	0, #87
wait_1sec_labelC0:
	djnz	0, wait_1sec_labelC0
	djnz	1, wait_1sec_labelC1
	djnz	2, wait_1sec_labelC2
	pop	2
	pop	1
	pop	0
	pop	PSW
	ret

;-----------------------------------------------
; call wait_a_ms
;-----------------------------------------------
; wartet "A" Millisekunden
;-----------------------------------------------
; input: A - milliseconds to wait
;-----------------------------------------------
; modifiziert: A=0
;-----------------------------------------------
; stack: 2 byte + caller address
;-----------------------------------------------

wait_a_ms:
	push	psw					; save used registers
	push	0
	mov	0, #181
wait_a_ms_labelA:
	nop
	nop
	nop
	djnz	0, wait_a_ms_labelA
	nop
	pop	0					; clean up stack (saved registers)
	pop	psw
	djnz	Acc, wait_a_ms
	ret

;-----------------------------------------------
; call crc8_add
;-----------------------------------------------
; calculates value in Acc into the running CRC
; in 'ram_checksum'
; using the table method. (faster but larger)
;-----------------------------------------------
; input: A            - value to add to CRC
;        ram_checksum - running CRC value
;-----------------------------------------------
; output: ram_checksum - updated CRC value
;-----------------------------------------------
; modified: CY
;-----------------------------------------------
; stack: 3 byte + caller address
;-----------------------------------------------

crc8_add:
	push	dph					; save used registers
	push	dpl
	push	acc

	mov	dptr, #crc8_data			; point to table
	xrl	a, ram_checksum			; XOR in CRC
	movc	a, @a+dptr				; get new CRC byte
	mov	ram_checksum, a			; store back

	pop	acc					; clean up stack (saved registers)
	pop	dpl
	pop	dph
	ret						; return to caller

crc8_data:
	db	000h, 05eh, 0bch, 0e2h, 061h, 03fh, 0ddh, 083h
	db	0c2h, 09ch, 07eh, 020h, 0a3h, 0fdh, 01fh, 041h
	db	09dh, 0c3h, 021h, 07fh, 0fch, 0a2h, 040h, 01eh
	db	05fh, 001h, 0e3h, 0bdh, 03eh, 060h, 082h, 0dch
	db	023h, 07dh, 09fh, 0c1h, 042h, 01ch, 0feh, 0a0h
	db	0e1h, 0bfh, 05dh, 003h, 080h, 0deh, 03ch, 062h
	db	0beh, 0e0h, 002h, 05ch, 0dfh, 081h, 063h, 03dh
	db	07ch, 022h, 0c0h, 09eh, 01dh, 043h, 0a1h, 0ffh
	db	046h, 018h, 0fah, 0a4h, 027h, 079h, 09bh, 0c5h
	db	084h, 0dah, 038h, 066h, 0e5h, 0bbh, 059h, 007h
	db	0dbh, 085h, 067h, 039h, 0bah, 0e4h, 006h, 058h
	db	019h, 047h, 0a5h, 0fbh, 078h, 026h, 0c4h, 09ah
	db	065h, 03bh, 0d9h, 087h, 004h, 05ah, 0b8h, 0e6h
	db	0a7h, 0f9h, 01bh, 045h, 0c6h, 098h, 07ah, 024h
	db	0f8h, 0a6h, 044h, 01ah, 099h, 0c7h, 025h, 07bh
	db	03ah, 064h, 086h, 0d8h, 05bh, 005h, 0e7h, 0b9h
	db	08ch, 0d2h, 030h, 06eh, 0edh, 0b3h, 051h, 00fh
	db	04eh, 010h, 0f2h, 0ach, 02fh, 071h, 093h, 0cdh
	db	011h, 04fh, 0adh, 0f3h, 070h, 02eh, 0cch, 092h
	db	0d3h, 08dh, 06fh, 031h, 0b2h, 0ech, 00eh, 050h
	db	0afh, 0f1h, 013h, 04dh, 0ceh, 090h, 072h, 02ch
	db	06dh, 033h, 0d1h, 08fh, 00ch, 052h, 0b0h, 0eeh
	db	032h, 06ch, 08eh, 0d0h, 053h, 00dh, 0efh, 0b1h
	db	0f0h, 0aeh, 04ch, 012h, 091h, 0cfh, 02dh, 073h
	db	0cah, 094h, 076h, 028h, 0abh, 0f5h, 017h, 049h
	db	008h, 056h, 0b4h, 0eah, 069h, 037h, 0d5h, 08bh
	db	057h, 009h, 0ebh, 0b5h, 036h, 068h, 08ah, 0d4h
	db	095h, 0cbh, 029h, 077h, 0f4h, 0aah, 048h, 016h
	db	0e9h, 0b7h, 055h, 00bh, 088h, 0d6h, 034h, 06ah
	db	02bh, 075h, 097h, 0c9h, 04ah, 014h, 0f6h, 0a8h
	db	074h, 02ah, 0c8h, 096h, 015h, 04bh, 0a9h, 0f7h
	db	0b6h, 0e8h, 00ah, 054h, 0d7h, 089h, 06bh, 035h

;-----------------------------------------------
; call crc8_add_to_r2
;-----------------------------------------------
; calculates value in Acc into the running CRC
; in R2
; using the table method. (faster but larger)
;-----------------------------------------------
; input: A  - value to add to CRC
;        R2 - running CRC value
;-----------------------------------------------
; output: R2 - updated CRC value
;-----------------------------------------------
; modified: CY
;-----------------------------------------------
; stack: 3 byte + caller address
;-----------------------------------------------

crc8_add_to_r2:
	push	dph					; save used registers
	push	dpl
	push	acc

	mov	dptr, #crc8_data			; point to table
	xrl	a, r2					; XOR in CRC
	movc	a, @a+dptr				; get new CRC byte
	mov	r2, a					; store back

	pop	acc					; clean up stack (saved registers)
	pop	dpl
	pop	dph
	ret						; return to caller

;-----------------------------------------------------------
; call calc_crc_and_store
;-----------------------------------------------------------
; adds value in A to running CRC value in R2, stores A at
; DPTR (external SRAM) and increments DPTR
;-----------------------------------------------------------
; input: A    - byte
;        R2   - running CRC value
;        DPTR - pointer to external SRAM
;-----------------------------------------------------------
; modified: DPTR++, R2, (ext. SRAM at old DPTR)=A
;-----------------------------------------------------------
; stack: caller address
;        + 5 byte (crc8_add_to_r2)
;-----------------------------------------------------------

calc_crc_and_store:
	call	crc8_add_to_r2			; calculate CRC
	movx	@dptr, a				; store A in ext. SRAM
	inc	dptr					; increase pointer
	ret						; return to caller


IFDEF _USE_DELAYS
;-----------------------------------------------
; call debug_wait
;-----------------------------------------------
; waits some time (7ms)
;-----------------------------------------------
; modified: N/A
;-----------------------------------------------
; stack: 1 byte + caller address
;        + 4 byte (wait_a_ms)
;-----------------------------------------------

debug_wait:
	push	acc
	mov	a, #7
	call	wait_a_ms
	pop	acc
	ret
ENDIF

