
;
;   ti-comm.inc
;
;   This file contains routines to communicate with a TI according to TI<->C LLP
;
;   last modified: 03/14/2002 mmddyy 23:22 by Burkart
;
; List of routines:
;
; - call wait_for_start_with_timeout
; - call wait_for_start_no_timeout
; - call receive_ti_packet
; - call receive_byte_from_ti
; - call check_received_ti_packet
; - call send_start
; - call send_packet_to_ti
; - call send_byte_to_ti
;


;-----------------------------------------------
; call wait_for_start_with_timeout
;-----------------------------------------------
; wait until START is received
; if a timeout occurs, CY is set, otherwise not
; handles complete START if appropriate
;-----------------------------------------------
; output: CY=1: timeout, START not received
;         CY=0: no timeout, START received
;-----------------------------------------------
; modified: N/A
;-----------------------------------------------
; stack: 2 byte + caller address
;-----------------------------------------------

wait_for_start_with_timeout:
	push	2					; save used registers
	push	3

	mov	r2, #0					; timeout loop (about 284ms)
wait_f_start_w_timeout_loop1a:
	mov	r3, #0
wait_f_start_w_timeout_loop1b:
	jnb	RED_WIRE, wait_for_start_with_timeout_ok1 ; red wire = 0V -> START
	djnz	r3, wait_f_start_w_timeout_loop1b
	djnz	r2, wait_f_start_w_timeout_loop1a

	; timeout

wait_for_start_with_timeout_timeout:
	pop	3					; clear up stack (saved registers)
	pop	2
	setb	c					; set CY in order to express error
	ret						; return to caller

wait_for_start_with_timeout_ok1:
	; START was received and now has to be handled

IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	clr	WHITE_WIRE				; white wire = 0V (START-ACK)

	mov	r2, #0					; timeout loop (about 284ms)
wait_f_start_w_timeout_loop2a:
	mov	r3, #0
wait_f_start_w_timeout_loop2b:
	jb	RED_WIRE, wait_for_start_with_timeout_ok2 ; red wire = 5V -> START-ACK-ACK
	djnz	r3, wait_f_start_w_timeout_loop2b
	djnz	r2, wait_f_start_w_timeout_loop2a

	; timeout

	jmp	wait_for_start_with_timeout_timeout ; handle timeout

wait_for_start_with_timeout_ok2:
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	WHITE_WIRE				; white wire = 5V (START-ACK-ACK-ACK)

	pop	3					; clear up stack (saved registers)
	pop	2
	clr	c					; clear CY
	ret						; return to caller

;-----------------------------------------------
; call wait_for_start_no_timeout
;-----------------------------------------------
; wait until START is received
; no timeout!!!
; handles complete START
;-----------------------------------------------
; modified: N/A
;-----------------------------------------------
; stack: caller address
;-----------------------------------------------

wait_for_start_no_timeout:
wait_for_start_no_timeout_wait1:
	jb	RED_WIRE, wait_for_start_no_timeout_wait1 ; wait until red wire = 0V -> START

	; START was received and now has to be handled

IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	clr	WHITE_WIRE				; white wire = 0V (START-ACK)

wait_for_start_no_timeout_wait2:
	jnb	RED_WIRE, wait_for_start_no_timeout_wait2 ; wait until red wire = 5V -> START-ACK-ACK

IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	WHITE_WIRE				; white wire = 5V (START-ACK-ACK-ACK)

	ret						; return to caller

;-----------------------------------------------
; call receive_ti_packet
;-----------------------------------------------
; Receives packet via TI<->C Protocol and
; stores it at ram_received_packet
; START must have been received in advance
;-----------------------------------------------
; modifiziert: N/A
;-----------------------------------------------
; Stack: 5 byte + caller address
;        + 5 byte (receive_byte_from_ti)
;-----------------------------------------------

receive_ti_packet:
	push	0					; save used registers
	push	2
	push	3
	push	4
	push	acc

	mov	P2, #(HIGH ext_tiuc_packet_in)	; initialize buffer pointer
	mov	r0, #(LOW ext_tiuc_packet_in)

	call	receive_byte_from_ti			; Byte empfangen
	movx	@r0, a					; und abspeichern
	inc	r0
	mov	r2, a					; Lnge (Lo-Byte) merken
	call	receive_byte_from_ti			; Byte empfangen
	movx	@r0, a					; und abspeichern
	inc	r0
	mov	r3, a					; Lnge (Hi-Byte) merken

	cjne	r3, #0, rcv_ti_packet_hi_not_zero
	cjne	r2, #0, rcv_ti_packet_hi_zero_lo_not_zero
	; Hi & Lo = 0

rcv_ti_packet_done:
	pop	acc					; clean up stack (saved registers)
	pop	4
	pop	3
	pop	2
	pop	0
	ret						; return to caller

rcv_ti_packet_hi_not_zero:
rcv_ti_packet_loop_hi:
	mov	r4, #0
rcv_ti_packet_loop_256:
	call	receive_byte_from_ti			; Byte empfangen
	movx	@r0, a					; und abspeichern
	inc	r0

	djnz	r4, rcv_ti_packet_loop_256
	djnz	r3, rcv_ti_packet_loop_hi

	cjne	r2, #0, rcv_ti_packet_loop_lo

	; Paketlnge(lo-byte)=0 -> Paket komplett empfangen

	jmp	rcv_ti_packet_done			; reception complete -> exit routine

rcv_ti_packet_hi_zero_lo_not_zero:
rcv_ti_packet_loop_lo:
	call	receive_byte_from_ti			; Byte empfangen
	movx	@r0, a					; und abspeichern
	inc	r0

	djnz	r2, rcv_ti_packet_loop_lo

	jmp	rcv_ti_packet_done			; reception complete -> exit routine

;-----------------------------------------------
; call receive_byte_from_ti
;-----------------------------------------------
; Empfngt ein Byte vom TI und legt es in A ab
; Bei Erfolg Rcksprung zum Aufrufer
; Bei Mierfolg Sprung zu reveive_byte_timedout
;-----------------------------------------------
; modifiziert: R0, R1=0, R2
;-----------------------------------------------
; stack: 3 byte + caller address
;-----------------------------------------------

receive_byte_from_ti:
	push	0
	push	1
	push	2

	mov	R1, #8					; 8 Bit wollen empfangen werden
receive_next_bit:
	mov	R0, #0					; warte auf Bit (ca. 284 ms)
wait_receive_next_bit1:
	mov	R2, #0
wait_receive_next_bit2:
	jnb	RED_WIRE, received_binary_one
	jnb	WHITE_WIRE, received_binary_zero
	djnz	R2, wait_receive_next_bit2
	djnz	R0, wait_receive_next_bit1

	; TI hat kein Bit gesendet (timeout)
	pop	2
	pop	1
	pop	0
	dec	SP					; clean up stack (caller address)
	dec	SP
	jmp	reveive_byte_timedout


received_binary_one:
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	clr	WHITE_WIRE				; sende ACK
	setb	c					; Carry-Flag = 1
	rlc	a					; empfangenes Bit = Carry-Flag = 1

	mov	R0, #0					; warte auf ACK-ACK (ca. 284 ms)
wait_received_binary_one1:
	mov	R2, #0
wait_received_binary_one2:
	jb	RED_WIRE, bit_binary_one_received
	djnz	R2, wait_received_binary_one2
	djnz	R0, wait_received_binary_one1

	; TI hat kein ACK-ACK gesendet (timeout)
	pop	2
	pop	1
	pop	0
	dec	SP					; clean up stack (caller address)
	dec	SP
	jmp	reveive_byte_timedout

bit_binary_one_received:
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	WHITE_WIRE				; Leitung wieder auf 5V bringen

	jmp	bit_almost_received			; warten, bis beide Leitungen wieder auf 5V liegen


received_binary_zero:
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	clr	RED_WIRE				; sende ACK
	clr	c					; Carry-Flag = 0
	rlc	a					; empfangenes Bit = Carry-Flag = 0

	mov	R0, #0					; warte auf ACK-ACK (ca. 284 ms)
wait_received_binary_zero1:
	mov	R2, #0
wait_received_binary_zero2:
	jb	WHITE_WIRE, bit_binary_zero_received
	djnz	R2, wait_received_binary_zero2
	djnz	R0, wait_received_binary_zero1

	; TI hat kein ACK-ACK gesendet (timeout)
	pop	2
	pop	1
	pop	0
	dec	SP					; clean up stack (caller address)
	dec	SP
	jmp	reveive_byte_timedout

bit_binary_zero_received:
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	RED_WIRE				; Leitung wieder auf 5V bringen

bit_almost_received:				; warten, bis beide Leitungen wieder auf 5V liegen
	mov	r0, #0
wait_bit_almost_received1:
	mov	r2, #0
wait_bit_almost_received2:
	jnb	RED_WIRE, wait_bit_almost_received3
	jb	WHITE_WIRE, bit_really_received
wait_bit_almost_received3:
	djnz	r2, wait_bit_almost_received2
	djnz	r0, wait_bit_almost_received1

bit_really_received:
	; beide Leitungen liegen auf 5V, bertragung des Bits abgeschlossen

	djnz	R1, receive_next_bit			; nchstes Bit empfangen

	pop	2
	pop	1
	pop	0
	ret						; zurck zum Aufrufer

;-----------------------------------------------
; call check_received_ti_packet
;-----------------------------------------------
; berprft ein zuvor via TI<->C empfangenes
; Paket anhand eines Templates auf Richtigkeit.
;-----------------------------------------------
; TODO: Support fr empfangene TI<->C-Pakete
;       > 256 byte
;-----------------------------------------------
; input: dptr - address of template packet
;        r2   - number of bytes which have to be
;               identical
;        received packet at #ram_received_packet
;-----------------------------------------------
; output: CY=0 - packets are identical
;         CY=1 - packets are different
;-----------------------------------------------
; modified: identical: dptr+=r2, r2=0
;           different: dptr, r2
;           any case: P2
;-----------------------------------------------
; stack: 1 byte + caller address
;-----------------------------------------------

check_received_ti_packet:
	push	0					; save used registers
	push	1

	mov	P2, #(HIGH ext_tiuc_packet_in)	; received packet
	mov	r0, #(LOW ext_tiuc_packet_in)

check_received_ti_packet_loop:
	movx	a, @r0					; get byte of received packet into r1
	mov	r1, a

	clr	a					; get byte of template packet into a
	movc	a, @a+dptr

	clr	c					; just make sure, CY doesn't interfere (damn boy)
	subb	a, r1					; identical?
	jnz	check_received_ti_packet_error	; no

	inc	dptr					; let dptr point to next byte of template packet
	inc	r0					; let r0 point to next byte of received packet

	djnz	r2, check_received_ti_packet_loop	; check all R2 byte

	; packet was OK

	pop	1					; clean up stack (saved register)
	pop	0
	clr	c					; clear CY
	ret						; return to caller

check_received_ti_packet_error:
	pop	1					; clean up stack (saved registers)
	pop	0
	setb	c					; set CY to acknowledge error
	ret						; return to caller

;------------------------------------------
; call send_start
;------------------------------------------
; sendet START an TI
; bei Erfolg Rcksprung zum Aufruf
; bei Timeout Sprung nach send_start_timedout
;------------------------------------------
; modifiziert: N/A
;------------------------------------------
; Stack: 2 Byte + caller address
;------------------------------------------

send_start:
	push	0
	push	1
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	clr	RED_WIRE				; sende START
	mov	R0, #0					; warte auf Besttigung (ca. 284ms)
wait_start_ack1:
	mov	R1, #0
wait_start_ack2:
	jnb	WHITE_WIRE, start_acked
	djnz	R1, wait_start_ack2
	djnz	R0, wait_start_ack1

	; TI hat nicht geantwortet (timeout)
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	RED_WIRE				; red wire wieder auf 5V setzen
	pop	1
	pop	0
	dec	SP					; clean up stack (caller address)
	dec	SP
	jmp	send_start_timedout

start_acked:
	; TI hat geantwortet
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	RED_WIRE				; red wire wieder auf 5V setzen

wait_start_end:					; warten auf white wire=5V
	jnb	WHITE_WIRE, wait_start_end

	; beide Leitungen sind auf 5V
	; bereit zu senden
	pop	1
	pop	0
	ret						; zurck zum Aufrufer

;-----------------------------------------------
; call send_packet_to_ti
;-----------------------------------------------
; sends a packet to the TI
;-----------------------------------------------
; input: dptr - address of packet to be sent
;               (in code memory)
;        r0   - number of byte to be sent
;-----------------------------------------------
; modified: dptr=dptr+r0, r0=0
;-----------------------------------------------
; stack: 1 byte + caller address
;        + 5 byte (send_byte_to_ti)
;-----------------------------------------------

send_packet_to_ti:
	push	acc					; save used register

send_packet_to_ti_loop:
	clr	a					; get next byte of packet into a
	movc	a, @a+dptr
	inc	dptr					; let dptr point to next byte of packet
	call	send_byte_to_ti			; send byte (in a) to TI
	djnz	r0, send_packet_to_ti_loop		; send all following bytes

	pop	acc					; clean up stack (saved register)
	ret						; return to caller

;-----------------------------------------------
; call send_byte_to_ti
;-----------------------------------------------
; sendet ein Byte in A an den TI
; bei Erfolg Rcksprung zum Aufruf
; bei Timeout Sprung nach send_byte_timedout
;-----------------------------------------------
; modifiziert: N/A
;-----------------------------------------------
; Stack: 3 Byte + caller address
;-----------------------------------------------

send_byte_to_ti:
	push	0					; save used registers
	push	1
	push	2
	push	PSW
	; zu bertragendes Byte in A
	mov	R1, #8					; 8 Bit wollen gesendet werden
send_next_bit:
	rlc	a					; nchstes zu sendendes Bit -> Carry-Flag
	jnc	send_binary_zero			; binre 1 oder 0?

	; sende binre 1
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	clr	RED_WIRE				; red wire = 0V
	mov	R0, #0					; warte auf Besttigung (ca. 284ms)
wait_one_ack1:
	mov	R2, #0
wait_one_ack2:
	jnb	WHITE_WIRE, one_sent			; ACK erhalten -> Leitungen wieder auf 5V setzen
	djnz	R2, wait_one_ack2
	djnz	R0, wait_one_ack1

	; TI hat kein ACK gesendet (timeout)
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	RED_WIRE				; red wire = 5V
	pop	PSW					; clean up stack (saved registers)
	pop	2
	pop	1
	pop	0
	dec	SP					; clean up stack (caller address)
	dec	SP
	jmp	send_byte_timedout

one_sent:
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	RED_WIRE				; red wire = 5V
wait_white_5v:					; warten bis white wire = 5V
	jnb	WHITE_WIRE, wait_white_5v

	; beide Leitungen wieder 5V
	
	sjmp	bit_sent				; djnz und nchstes Bit senden

send_binary_zero:
	; sende binre 0
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	clr	WHITE_WIRE				; white wire = 0V
	mov	R0, #0					; warte auf Besttigung (ca. 284ms)
wait_zero_ack1:
	mov	R2, #0
wait_zero_ack2:
	jnb	RED_WIRE, zero_sent			; ACK erhalten -> Leitungen wieder auf 5V setzen
	djnz	R2, wait_zero_ack2
	djnz	R0, wait_zero_ack1

	; TI hat kein ACK gesendet (timeout)
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	WHITE_WIRE				; white wire = 5V
	pop	PSW					; clean up stack (saved registers)
	pop	2
	pop	1
	pop	0
	dec	SP					; clean up stack (caller address)
	dec	SP
	jmp	send_byte_timedout

zero_sent:
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	WHITE_WIRE				; white wire = 5V
wait_red_5v:						; warten bis red wire = 5V
	jnb	RED_WIRE, wait_red_5v

	; beide Leitungen wieder 5V

bit_sent:
	djnz	R1, send_next_bit			; nchstes Bit senden

	pop	PSW					; clean up stack (saved registers)
	pop	2
	pop	1
	pop	0
	ret						; zurck zum Aufrufer

