; Autor    : Burkart
; Projekt  : TInet Client C
; Quarz    : 11.0592 MHz
; Hardware : 32kB externes SRAM (62256)
;            TxD & RxD
;            Red wire & white wire siehe Quellcode
;            nRF-Pins siehe Quellcode
;            kompatibel zur Platine_2.0
; C-Typ   : AT89C51 getestet, theoretisch auch AT89C52
; Generator: CodeGen Stable Version 1.0, Erik Buchmann '01
; Assembler: ASEM51 (Version 1.2 getestet)
; Datum    : 09.02.2002 - 18.03.2002
;            last modified: 03/19/2002 mmddyy 09:55
; Funktion : Relay/Filter zwischen TI und nRF401 bzw. PC
;            - LED show (off)
;            - Login gem TI<->C HLP
;            - Login gem radio HLP
;            - Kommunikation mit TI gem TI<->C HLP
;            - Kommunikation per Funk gem radio HLP
;            - Logout gem TI<->C HLP
;            - Logout gem radio HLP
; Status   : In der Entwicklung; lauffhig
;---------------------------------------------------------------
$NOPAGING
$PAGEWIDTH(200)
$NOTABS

RED_WIRE	BIT P1.6	; red wire of TI cable
WHITE_WIRE	BIT P1.7	; white wire of TI cable
TXEN_PIN	BIT P1.4	; TXEN of nRF401 - 1=transmit, 0=receive
CS_PIN		BIT P1.5	; CS of nRF401 - 0=Channel 1 (433.92 MHz), 1=Channel 2 (434.33 MHz)
PWR_UP_PIN	BIT P1.3	; PWR_UP of nRF401 - 1=operating, 0=standby
LED_PIN	BIT P1.2	; LED on TI_Net V2.0 Board - 1=on, 0=off


UCTYPE EQU 51h	; C type (here: AT89C51)
SRAM   EQU 32h	; ext. SRAM size (here: 32kB)
HI_VER EQU 01h	; C software hi-version
LO_VER EQU 05h	; C software lo-version
; Version: 1.05


START_CHAR EQU 'U'			; send this char (and an FFh in advance) before each packet
_CHANNEL EQU 2			; = 1 -> channel 1 (433.92 MHz); else -> channel 2 (434.33 MHz)

;;; _USE_DELAYS EQU 1		; define this to wait before each red/white wire transition
_DI_DURING_TIUC_COMM EQU 1		; define this to disable interrupts during TI<->C communication
_TXD_DEFAULT_LO EQU 1		; define this to default TxD to 0V, otherwise TxD defaults to 5V
;;; _USER_LED_ON_SER_INT EQU 1	; define this to flash user LED during serial interrupt
_USER_LED_ON_CHECKSUM_OK EQU 1	; define this to turn on user LED after each packet with correct checksum
_SHOW_DATA_AFTER_EMPTY EQU 1	; define this to transmit radio packets to TI only after an additional {empty}
;;; _STANDARD_ACK_LOGIN EQU 1	; define this to send a standard {ACK} to acknowledge a received


$INCLUDE(var_buf.inc)	; variable and buffer definitions

;===========================================================
; program start
;===========================================================

 ORG 0h
	jmp	start					; jump over interrupt


 ORG 23h
$INCLUDE(serint.inc)				; serial interrupt handler

;===========================================================

program_start:
start:
	; initialize critical bits:

	clr	TXEN_PIN				; make sure we don't send bullshit while showing off
	setb	PWR_UP_PIN				; needed to have RxD-LED lit

	; show off (about 2.6 seconds):
	; LEDs: x x x o x x (P1.2, RxD, TxD, Ch0, Ch1, PWR)
	; LEDs: x o x o x x:
	clr	RXD
	clr	a
	call	wait_a_ms
	; LEDs: o x o x o x:
	clr	LED_PIN
	setb	RXD
	clr	TXD
	clr	CS_PIN
	call	wait_a_ms
	; LEDs: x o x o x x:
	setb	LED_PIN
	clr	RXD
	setb	TXD
	setb	CS_PIN
	call	wait_a_ms
	; LEDs: o o o o x x:
	clr	LED_PIN
	clr	TXD
	call	wait_a_ms
	; LEDs: o o o x o x:
	clr	CS_PIN
	call	wait_a_ms
	; LEDs: o o x o x x:
	setb	TXD
	setb	CS_PIN
	call	wait_a_ms
	; LEDs: o x o x o x:
	setb	RXD
	clr	TXD
	clr	CS_PIN
	call	wait_a_ms
	; LEDs: x o x o x x:
	setb	LED_PIN
	clr	RXD
	setb	TXD
	setb	CS_PIN
	call	wait_a_ms
	mov	r0, #3
main_show_off:
	; LEDs: o x o x o x:
	clr	LED_PIN
	setb	RXD
	clr	TXD
	clr	CS_PIN
	mov	a, #100
	call	wait_a_ms
	; LEDs: x o x o x x:
	setb	LED_PIN
	clr	RXD
	setb	TXD
	setb	CS_PIN
	mov	a, #100
	call	wait_a_ms
	djnz	r0, main_show_off

	; initalize I/O pins, flags and so on:

	setb	RXD					; make it possible to receive something
IFDEF _TXD_DEFAULT_LO
	clr	TXD					; TxD low by default (spacing condition?)
ELSE
	setb	TXD					; TxD high by default (marking condition?)
ENDIF
	clr	TXEN_PIN				; TXEN auf Empfangen setzen
IF (_CHANNEL = 1)
	clr	CS_PIN					; Channel 1 (433.92 MHz) [NOT: Channel 2 (434.33 MHz)]
ELSE
	setb	CS_PIN					; Channel 2 (434.33 MHz) [NOT: Channel 1 (433.92 MHz)]
ENDIF
	setb	PWR_UP_PIN				; operation mode
	clr	LED_PIN				; LED off
	setb	RED_WIRE ; NOT NECESSARY?
	setb	WHITE_WIRE ; NOT NECESSARY?

	clr	bit_radio_ti_packet_pending		; 1=pending radio packet which has to be sent
	clr	bit_radio_ti_packet_sent		; 1=radio packet has been sent
	clr	bit_radio_packet_rcvd		; 1=radio packet with RCPT=me received, at ext_radio_packet_in
	clr	bit_radio_reply_ack			; 1=ACK reply appropriate
	clr	bit_radio_reply_nack			; 1=NACK reply appropriate
	setb	bit_int_awaiting_start		; 1=awaiting START-char, 0=START-char already received
	clr	bit_radio_logged_out			; 1=logged out (radio), 0=logged in (radio) / not yet logged in (radio)
	clr	bit_radio_packet_rcvd_completely	; [...]

	mov	ram_radio_packet_l_in_offset, #0
	mov	ram_radio_packet_l_in_remaining, #0
	mov	ram_radio_byte_to_send_offset, #0	; next byte to send, offset to ext_radio_packet_out
	mov	ram_radio_byte_to_send, #0		; remaining byte to send
	mov	ram_all_data_offset_hi, #0		; offset of next byte in ext_radio_packet_all_data (hi-byte)
	mov	ram_all_data_offset_lo, #0		; offset of next byte in ext_radio_packet_all_data (lo-byte)


; serielle Schnittstelle aktivieren
;  19200 Baud, Modus 1
	mov	TL1, #253
	mov	TH1, #253

; die SFR's initialisieren
	mov	SP, #30h
	mov	SCON, #01010010b
		; Serielle Schnittstelle:
		; Mode 1 (8-Bit UART)
		; RI wird auch aktiviert, wenn kein gltiges Stopbit empfangen wurde
		; Reception enabled
	mov	PCON, #10000000b
		; Doppelte Baudrate fr UART
		; General purpose flag bits GF1 & GF0 = 0
		; Power down & Idle mode aus
	mov	TMOD, #00100000b
		; Timer 1:
		; Software control
		; Timer operation (input from internal system clock)
		; 8-bit Auto-Reload Timer
		; Timer 0:
		; Software control
		; Timer operation (input from internal system clock)
		; 13-bit Timer
	mov	TCON, #01000000b
		; Timer 1 an (wird zur Baudratenerzeugung bentigt)
	mov	IE, #10000000b
		; Interrupts generell zugelassen (EA=1)
		; alle Interrupts aus

	; verwendet 32kB-SRAM (62256)
	; P2.7 = 0
	mov	P2, #0


	; try to send START (fails if TI is off(line)):

IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	clr	RED_WIRE				; sende START
	mov	r1, #0					; warte auf Besttigung (ca. 284ms)
main_wait_start_ack1:
	mov	r2, #0
main_wait_start_ack2:
	jnb	WHITE_WIRE, main_start_acked
	djnz	r2, main_wait_start_ack2
	djnz	r1, main_wait_start_ack1

	; TI hat nicht geantwortet (timeout)
	; TI seems to be off(line)
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	RED_WIRE				; red wire wieder auf 5V setzen
	jmp	wait_for_ti				; wait for TI to send <TI started>

main_start_acked:
	; TI hat geantwortet
IFDEF _USE_DELAYS
	call	debug_wait ; DEBUG CODE
ENDIF
	setb	RED_WIRE				; red wire wieder auf 5V setzen

main_wait_start_end:				; warten auf white wire=5V
	jnb	WHITE_WIRE, main_wait_start_end

	; beide Leitungen sind auf 5V
	; bereit zu senden

	; send to TI: <C (re) started>

	mov	dptr, #tiucpacket_uc_re_started	; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_packet_to_ti			; send packet to TI

	jmp	receive_ti_software_version		; go on to receiving <TI software version>

wait_for_ti:
	; wait for <TI started>:

	call	wait_for_start_no_timeout		; wait until START
	call	receive_ti_packet			; let's see, what the TI is sending

	mov	dptr, #tiucpacket_ti_started	; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; verify received packet
	jc	to1_soft_reset			; packets different -> soft reset

	; TI sent <TI started>

	; send <C ACK>:

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI

	; TI has to send <TI software version>:

receive_ti_software_version:
	call	wait_for_start_with_timeout		; wait for START
;	jc	to1_start_received_timedout		; timeout -> soft-reset
	jc	wait_for_ti				; timeout -> wait for TI
	call	receive_ti_packet			; let's see, what the TI is sending

	mov	dptr, #tiucpacket_ti_software_version ; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; verify received packet
;	jc	to1_soft_reset			; packets different -> soft-reset
	jc	wait_for_ti				; packets different -> wait for TI

	; TI sent <TI software version>

	mov	P2, #(HIGH ext_tiuc_packet_in)	; get to first byte of attached data
	mov	r0, #(LOW ext_tiuc_packet_in + 3)
	movx	a, @r0					; save calc type
	mov	ram_calc_type, a
	inc	r0
	movx	a, @r0					; save TI hi version
	mov	ram_ti_hi_version, a
	inc	r0
	movx	a, @r0					; save TI lo version
	mov	ram_ti_lo_version, a

	; send <C software version>

	mov	dptr, #tiucpacket_uc_software_version ; template packet
	mov	r0, #7					; consists of 7 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI

	; receive <TI login data>

	call	wait_for_start_no_timeout		; wait for START
	call	receive_ti_packet			; let's see, what the TI is sending

	mov	dptr, #tiucpacket_ti_login_data	; template packet
	mov	r2, #3					; first 3 byte have to be identical
	call	check_received_ti_packet		; verify received packet
	jc	to1_soft_reset			; packets different -> soft-reset

	; save username:

	mov	r2, #8					; username is 8 byte long
	mov	r1, #ram_login_name			; place to store username at
	mov	P2, #(HIGH ext_tiuc_packet_in)	; get to first byte of attached data
	mov	r0, #(LOW ext_tiuc_packet_in + 3)

copy_login_name:
	movx	a, @r0					; copy username byte-wise
	mov	@r1, a

	inc	r0					; advance to next byte
	inc	r1
	djnz	r2, copy_login_name			; copy all 8 byte of username

	; send <C logging in>

	mov	dptr, #tiucpacket_uc_logging_in	; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI

	; receive <TI ACK>

	call	wait_for_start_with_timeout		; wait for START
to1_start_received_timedout:
	jc	to3_start_received_timedout		; timeout -> soft-reset
	call	receive_ti_packet			; let's see, what the TI is sending

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r2, #3					; consists of 3 byte
	call	check_received_ti_packet		; verify received packet
to1_soft_reset:
	jc	to3_soft_reset			; packets different -> soft-reset

	jmp	radio_login_start


to3_start_received_timedout:
	jmp	start_received_timedout
to3_soft_reset:
	jmp	soft_reset


$INCLUDE(radlogin.inc)				; perform radio login


	; send <C logged in>

	mov	dptr, #tiucpacket_uc_logged_in	; template packet
	mov	r0, #3					; consists of 3 byte
	call	send_start				; send START to TI
	call	send_packet_to_ti			; send packet to TI

	; receive <TI ACK>

	call	wait_for_start_with_timeout		; wait for START
to2_start_received_timedout:
	jc	start_received_timedout		; timeout -> soft-reset
	call	receive_ti_packet			; let's see, what the TI is sending

	mov	dptr, #tiucpacket_ack		; template packet
	mov	r2, #3					; consists of 3 byte
	call	check_received_ti_packet		; verify received packet
to2_soft_reset:
	jc	soft_reset				; packets different -> soft-reset

IFDEF _DI_DURING_TIUC_COMM
	setb	IE.4					; enable serial interrupt
ENDIF

main_loop:
IFDEF _DI_DURING_TIUC_COMM
	setb	IE.7					; enable all interrupts
ENDIF

	jnb	RED_WIRE, to1_main_handle_start	; START -> let's see what it is

	; Auf "Events" des Interrupt handlers reagieren:

	jb	bit_radio_logged_out, handle_radio_logged_out ; radio logged out
	jb	bit_radio_packet_rcvd, handle_radio_packet_rcvd ; radio packet received
	jb	bit_radio_ti_packet_sent, handle_radio_ti_packet_sent ; radio TI packet sent
	jnb	bit_rcvd_radio_empty, dont_handle_radio_empty_rcvd
	jmp	handle_radio_empty_rcvd		; {empty} received
dont_handle_radio_empty_rcvd:

	jmp	main_loop

to1_main_handle_start:
	jmp	main_handle_start
;===============================================

send_start_timedout:				; Timeout beim Senden von START
send_byte_timedout:					; Timeout beim Senden eines Bits (eines Bytes)
start_received_timedout:				; Timeout beim Empfangen von START
reveive_byte_timedout:				; Timeout beim Empfangen eines Bits (eines Bytes)
soft_reset:

	call	wait_1sec				; wait 1 second
	jmp	program_start				; soft reset


$INCLUDE(events.inc)	; handle interrupt events during main loop

$INCLUDE(handleti.inc)	; handle <TI ...> during main loop

$INCLUDE(ti-comm.inc)	; TI<->C communication

$INCLUDE(serialio.inc)	; serial I/O

$INCLUDE(supply.inc)	; supply routines

$INCLUDE(packtdef.inc)	; packet definitions

$INCLUDE(progend.inc)	; program end w/ size check
