
;
;   var_buf.inc
;
;   This file contains variable and buffer definitions
;
;   last modified: 03/19/2002 mmddyy 09:53 by Burkart
;


;-----------------------------------------------------------
; internal RAM
;-----------------------------------------------------------
; AT89C51: 128 byte
; AT89C52: additional 128 byte $80..$FF (indirect addressing)
;-----------------------------------------------------------
; 00h..07h  register bank 0
; 08h..1Fh  register banks 1, 2, 3
; 20h..2Fh  bit addressable segment
; 30h..??h  stack
; 30h..7Fh  scratch pad area
;-----------------------------------------------------------

ram_calc_type		DATA 7Fh	; 7Fh       1 byte  calc type of TI
ram_ti_hi_version		DATA 7Eh	; 7Eh       1 byte  TI software version (hi)
ram_ti_lo_version		DATA 7Dh	; 7Dh       1 byte  TI software version (lo)
ram_login_name		DATA 75h	; 75h..7Ch  8 byte  login name

ram_radio_byte_to_send_offset DATA 74h	; 74h       1 byte  next byte to send, offset to ext_radio_packet_out
ram_radio_byte_to_send	DATA 73h	; 73h       1 byte  remaining byte to send

ram_radio_address		DATA 72h	; 72h       1 byte  radio address of this UU

ram_checksum			DATA 71h	; 71h       1 byte  running checksum byte
ram_int_checksum		DATA 70h	; 70h       1 byte  running checksum byte, use only in serial_int!

ram_radio_in_content_id	DATA 6Fh	; 6Fh       1 byte  Content-ID of packet at ext_radio_packet_all_data

ram_all_data_offset_hi	DATA 6Eh	; 6Eh       1 byte  offset of next byte in ext_radio_packet_all_data (hi-byte)
ram_all_data_offset_lo	DATA 6Dh	; 6Dh       1 byte  offset of next byte in ext_radio_packet_all_data (lo-byte)

ram_radio_packet_l_in_remaining DATA 6Ch	; 6Ch       1 byte  remaining byte until packet at ext_radio_packet_last_in is complete
ram_radio_packet_l_in_offset DATA 6Bh	; 6Bh       1 byte  offset to ext_radio_packet_last_in for next received byte

ram_multipckt_last_packet	DATA 6Ah	; 6Ah       1 byte  number of last single packet (of multi-packet) received
ram_multipckt_total_packets DATA 69h	; 69h       1 byte  number of total packets (of multi-packet)

bit_radio_ti_packet_pending BIT 20h.0	; 20h.0     1 bit   1=pending radio packet which has to be sent
bit_radio_ti_packet_sent	BIT 20h.1	; 20h.1     1 bit   1=radio packet has been sent
bit_radio_packet_rcvd	BIT 20h.2	; 20h.2     1 bit   1=radio packet with RCPT=me received, at ext_radio_packet_in
bit_radio_reply_ack		BIT 20h.3	; 20h.3     1 bit   1=ACK reply appropriate
bit_radio_reply_nack	BIT 20h.4	; 20h.4     1 bit   1=NACK reply appropriate
bit_int_awaiting_start	BIT 20h.5	; 20h.5     1 bit   1=awaiting START-char, 0=START-char already received
bit_radio_logged_out	BIT 20h.6	; 20h.6     1 bit   1=logged out (radio), 0=logged in (radio) / not yet logged in (radio)
bit_rcvd_radio_empty	BIT 20h.7	; 20h.7     1 bit   1=received {empty}, has to be reset by main program
bit_radio_packet_rcvd_completely BIT 21h.0 ; 21h.8  1 bit   1=received complete radio (multi-)packet

;-----------------------------------------------------------
; external SRAM
;-----------------------------------------------------------
; 62256: 32kB
;-----------------------------------------------------------
; P2.7=0: SRAM enabled
; P2.7=1: SRAM stand-by
;-----------------------------------------------------------
; randomly accessible by 'movx a, @dptr' or 'movx @dptr, a'
; paged access by 'mov p2, #addr_hi' plus 'movx a, @ri' or
; 'movx @ri, a'
;-----------------------------------------------------------

ext_tiuc_packet_in		XDATA 0000h	; 0000h..01FFh  512 byte  last via TI<->C received packet
ext_radio_packet_out	XDATA 0200h	; 0200h..02FFh  256 byte  radio packet, TI wishes to send to TI-Server
ext_radio_packet_last_in	XDATA 0300h	; 0300h..03FFh  256 byte  last received single radio packet (RCPT needn't be = me)
ext_radio_packet_in		XDATA 0400h	; 0400h..04FFh  256 byte  last received single radio packet with RCPT=me
ext_radio_packet_out_int	XDATA 0500h	; 0500h..05FFh  256 byte  packet which has to be sent by interrupt handler (TI)
ext_radio_packet_all_data	XDATA 1000h	; 1000h..7FFFh  28 kbyte  maybe assembled data part of received radio (multi-)packet
ext_last_byte		XDATA 7FFFh   ; last byte of ext. SRAM for 62256 (32kB)

